/*
 * spi.c
 *
 *  Created on: 01.09.2015
 *      Author: dmitriy.dzhevaga
 */

#include "spi.h"

/******************************************************************************/
/*-------------------------Function Implementations---------------------------*/
/******************************************************************************/

void spiModuleEnable(SpiModule spiModule)
{
	uint16 endinit_pw = IfxScuWdt_getCpuWatchdogPassword();
	IfxScuWdt_clearCpuEndinit(endinit_pw);
	((Ifx_QSPI*)spiModule)->CLC.U = 0;
	volatile uint32 dummy = ((Ifx_QSPI*)spiModule)->CLC.U;
	IfxScuWdt_setCpuEndinit(endinit_pw);
}

void spiModuleClockDividerConfig(SpiModule spiModule, uint8 moduleClockDivider)
{
	((Ifx_QSPI*)spiModule)->GLOBALCON.B.TQ = moduleClockDivider - 1;
	((Ifx_QSPI*)spiModule)->GLOBALCON.B.MS = 0; // Master Transmit and Receive
	((Ifx_QSPI*)spiModule)->GLOBALCON.B.EN = 1; // RUN requested
}

void spiChannelConfig(SpiModule spiModule, SpiChannel spiChannel, SpiMode spiMode, SpiChannelClockDivider *spiChannelClockDivider, SpiPinsMapping *spiPinsMapping)
{
	IfxPort_setPinModeOutput(spiPinsMapping->sls.port, spiPinsMapping->sls.pinIndex, IfxPort_OutputMode_pushPull, spiPinsMapping->sls.altFunct); 	 // QSPI output SLS
	IfxPort_setPinModeOutput(spiPinsMapping->sclk.port, spiPinsMapping->sclk.pinIndex, IfxPort_OutputMode_pushPull, spiPinsMapping->sclk.altFunct); // QSPI output SCLK
	IfxPort_setPinModeOutput(spiPinsMapping->mtsr.port, spiPinsMapping->mtsr.pinIndex, IfxPort_OutputMode_pushPull, spiPinsMapping->mtsr.altFunct); // QSPI output MTSR
	IfxPort_setPinModeInput(spiPinsMapping->mrst.port, spiPinsMapping->mrst.pinIndex, IfxPort_InputMode_noPullDevice); 							 // QSPI input MRST

	Ifx_QSPI* ifxSpiModule = (Ifx_QSPI*)spiModule;
	ifxSpiModule->ECON[spiChannel & 0x07].B.CPOL = spiMode >> 1;
	ifxSpiModule->ECON[spiChannel & 0x07].B.CPH = spiMode & 1;
	ifxSpiModule->ECON[spiChannel & 0x07].B.Q = spiChannelClockDivider->Q - 1;
	ifxSpiModule->ECON[spiChannel & 0x07].B.A = spiChannelClockDivider->ABC >> 8;
	ifxSpiModule->ECON[spiChannel & 0x07].B.B = spiChannelClockDivider->ABC >> 4;
	ifxSpiModule->ECON[spiChannel & 0x07].B.C = spiChannelClockDivider->ABC & 0x0F;

	ifxSpiModule->SSOC.B.OEN = 1 << spiChannel; // Enable Bits for the SLSO Outputs
}

inline static void spiWaitBusy(Ifx_QSPI* spiModule) {
	while(spiModule->STATUS.B.PHASE != 0) {
		;
	}
}

void spiExchange(SpiModule spiModule, SpiChannel spiChannel, SpiShiftMode spiShiftMode, void *wbuf, void *rbuf, uint16 len)
{
	if(len == 0) {
		return;
	}

	Ifx_QSPI* ifxSpiModule = (Ifx_QSPI*)spiModule;

	if(len == 1) {
		// send 8 bits
		ifxSpiModule->BACONENTRY.U = (spiChannel << 28/*CS*/) |
									 (spiShiftMode << 21/*MSB*/) |
									 (1 << 0/*LAST*/) |
									 (7 << 23/*DL*/);
	}
	else if(len <= 32) {
		// send N bytes
		ifxSpiModule->BACONENTRY.U = (spiChannel << 28/*CS*/) |
									 (spiShiftMode << 21/*MSB*/) |
									 (1 << 0/*LAST*/) |
									 ((len - 1) << 23/*DL*/) |
									 (1 << 22/*BYTE*/);
	} else {
		// send N bytes with Extended Data Length feature
		// TODO: XXLCON is not accessible
		ifxSpiModule->XXLCON.B.XDL = len - 1;
		ifxSpiModule->BACONENTRY.U = (spiChannel << 28/*CS*/) |
									 (spiShiftMode << 21/*MSB*/) |
									 (1 << 0/*LAST*/) |
									 (1 << 22/*BYTE*/);
	}

	while(len > 0) {

		uint32 buf = 0;

		uint8 bytesToBuf = len > 4 ? 4 : len; // not more then 4 bytes per iteration

		if(spiShiftMode == LsbFirst) { // copy bytes to the 32 bit length transmitter buffer depending on the alignment
			for(sint8 i = 0; i < bytesToBuf; i++) {
				((uint8 *)&buf)[i] = *((uint8 *)wbuf++);
			}
		} else {
			for(sint8 i = bytesToBuf - 1; i >= 0; i--) {
				((uint8 *)&buf)[i] = *((uint8 *)wbuf++);
			}
		}

		ifxSpiModule->DATAENTRY[spiChannel].U = buf; // write to transmitter buffer
		spiWaitBusy(ifxSpiModule); // wait for complete
		buf = ifxSpiModule->RXEXIT.U; // read receiver buffer

		if(spiShiftMode == LsbFirst) { // copy bytes from the 32 bit length receiver buffer depending on the alignment
			for(sint8 i = 0; i < bytesToBuf; i++) {
				*((uint8 *)rbuf++) = ((uint8 *)&buf)[i];
			}
		} else {
			for(sint8 i = bytesToBuf - 1; i >= 0; i--) {
				*((uint8 *)rbuf++) = ((uint8 *)&buf)[i];
			}
		}

		len -= bytesToBuf;
	}
}

/*
void spiTest(void)
{
	spiModuleEnable(Spi0);
	spiModuleClockDividerConfig(Spi0, 50);

	SpiPinsMapping spiPinMapping = {
		{&MODULE_P33, 5, IfxPort_OutputIdx_alt2},  // QSPI0 output SLS7
		{&MODULE_P20, 11, IfxPort_OutputIdx_alt3}, // QSPI0 output SCLK0
		{&MODULE_P20, 14, IfxPort_OutputIdx_alt3}, // QSPI0 output MTSR0
		{&MODULE_P20, 12} 						   // QSPI0 input MRST0A
	};
	SpiChannelClockDivider spiChannelClockDivider = {div4, 2};
	spiChannelConfig(Spi0, Ch7, Mode0, &spiChannelClockDivider, &spiPinMapping);

	uint8 wbuf[2] = {0x05, 0x00};
	uint8 rbuf[2];
	while(1) {
		spiExchange(Spi0, Ch7, MsbFirst, wbuf, rbuf, 2);
		pauseInSec(0.01);
	}
}
*/
