/*
 * spi.h
 *
 *  Created on: 01.09.2015
 *      Author: dmitriy.dzhevaga
 */

#ifndef SPI_H_
#define SPI_H_

/******************************************************************************/
/*----------------------------------Includes----------------------------------*/
/******************************************************************************/
#include "Cpu/Std/Ifx_Types.h"
#include "Scu/Std/IfxScuWdt.h"
#include "IfxScu_reg.h"
#include "IfxSrc_reg.h"
#include "Src/Std/IfxSrc.h"
#include "IfxQspi_reg.h"
#include "Port/Io/IfxPort_Io.h"

/******************************************************************************/
/*--------------------------------Enumerations--------------------------------*/
/******************************************************************************/
typedef enum {
	Spi0 = (uint32)&MODULE_QSPI0,
	Spi1 = (uint32)&MODULE_QSPI1,
	Spi2 = (uint32)&MODULE_QSPI2,
	Spi3 = (uint32)&MODULE_QSPI3
}SpiModule;

typedef enum {
	Ch0 = 0,
	Ch1 = 1,
	Ch2 = 2,
	Ch3 = 3,
	Ch4 = 4,
	Ch5 = 5,
	Ch6 = 6,
	Ch7 = 7,
	Ch8 = 8,
	Ch9 = 9,
	Ch10 = 10,
	Ch11 = 11,
	Ch12 = 12,
	Ch13 = 13,
	Ch14 = 14,
	Ch15 = 15
}SpiChannel;

typedef enum {
    Mode0 = 0, /**< Mode 0: CPOL = 0 - Active-high clocks selected. CPHA = 0 - Sampling of data occurs at odd edges of the SCK clock. */
    Mode1 = 1, /**< Mode 1: CPOL = 0 - Active-high clocks selected. CPHA = 1 - Sampling of data occurs at even edges of the SCK clock. */
    Mode2 = 2, /**< Mode 2: CPOL = 1 - Active-low clocks selected. CPHA = 0 - Sampling of data occurs at odd edges of the SCK clock. */
    Mode3 = 3  /**< Mode 3: CPOL = 1 - Active-low clocks selected. CPHA = 1 - Sampling of data occurs at even edges of the SCK clock. */
} SpiMode;

typedef enum {
    LsbFirst = 0, /**< Shift LSB first */
    MsbFirst = 1  /**< Shift MSB first */
} SpiShiftMode;

/** Clock divider ABC is calculated as (A+1)+B+C. If B=0, than C is minimum 1 per hardware.
 *
 *  Total channel clock divider ABC*Q (see ::SpiChannelClockDivider) must be not less then 4. To simplify configuration ABC dividers less then 4 is excluded from enumeration.
*/
typedef enum {
	//div2  = 0x001, /**< A=0, B=0, C=1 */
	//div3  = 0x002, /**< A=0, B=0, C=2 */
    div4  = 0x003, /**< A=0, B=0, C=3 */
    div5  = 0x013, /**< A=0, B=1, C=3 */
    div6  = 0x023, /**< A=0, B=2, C=3 */
    div7  = 0x033, /**< A=0, B=3, C=3 */
    div8  = 0x133, /**< A=1, B=3, C=3 */
    div9  = 0x233, /**< A=2, B=3, C=3 */
    div10 = 0x333  /**< A=3, B=3, C=3 */
} SpiABCClockDivider;

/******************************************************************************/
/*---------------------------------Structures---------------------------------*/
/******************************************************************************/

typedef struct _SpiPinOutConfig {
	Ifx_P* port;				/**< Port */
	uint8 pinIndex;				/**< Pin number */
	IfxPort_OutputIdx altFunct; /**< Pin alternate function index */
} SpiPinOutConfig;

typedef struct _SpiPinInConfig {
	Ifx_P* port;	/**< Port */
	uint8 pinIndex;	/**< Pin number */
} SpiPinInConfig;

typedef struct _SpiPinMapping {
	SpiPinOutConfig sls; 	/**< slave select signal SLS */
	SpiPinOutConfig sclk; 	/**< serial clock SCLK */
	SpiPinOutConfig mtsr;	/**< data in master to slave direction MTSR (Master Transmit Slave Receive) */
	SpiPinInConfig mrst;	/**< data in slave to master direction MRST (Master Receive Slave Transmit) */
} SpiPinsMapping;

typedef struct _SpiChannelClockDivider {
	SpiABCClockDivider ABC;
	unsigned int Q:6;
} SpiChannelClockDivider;

/******************************************************************************/
/*-------------------------Global Function Prototypes-------------------------*/
/******************************************************************************/

/**@brief Enable module clock.
 *
 * @param spiModule SPI module to enable
 * @return None
 *
 * Coding example:
 * @code
 *     spiModuleEnable(Spi0);
 * @endcode
 *
 */
extern void spiModuleEnable(SpiModule spiModule);

/**@brief Set module clock divider.
 *
 * Effective channel divider is calculated from module and channel clock dividers: Fclk = Fbaund2/(moduleClockDivider*channelClockDividerABC*channelClockDividerQ), see ::spiChannelConfig
 *
 * @param spiModule module to configure
 * @param moduleClockDivider divider value (1-255)
 * @return None
 *
 * Coding example:
 * @code
 *     spiModuleClockDividerConfig(Spi0, 100);
 * @endcode
 *
 */
extern void spiModuleClockDividerConfig(SpiModule spiModule, uint8 moduleClockDivider);

/**@brief Configure selected channel.
 *
 * Effective channel divider is calculated from module and channel clock dividers: Fclk = Fbaund2/(moduleClockDivider*channelClockDividerABC*channelClockDividerQ), see ::spiModuleEnable
 *
 * The 16 slave select channels are subdivided in two groups of 8 slave select channels. That means channel 8 has the same settings as channel 0, channel 9 as channel 1, and so on.
 *
 * @param spiModule module to configure
 * @param spiChannel channel to configure
 * @param spiMode spi mode
 * @param spiChannelClockDivider channel dividers
 * @param spiPinsMapping channel pins mapping
 * @return None
 *
 * Coding example:
 * @code
 *     SpiPinsMapping spiPinMapping = {
 *			{&MODULE_P33, 5, IfxPort_OutputIdx_alt2},  // QSPI0 output SLS7
 *			{&MODULE_P20, 11, IfxPort_OutputIdx_alt3}, // QSPI0 output SCLK0
 *			{&MODULE_P20, 14, IfxPort_OutputIdx_alt3}, // QSPI0 output MTSR0
 *			{&MODULE_P20, 12} 						   // QSPI0 input MRST0A
 *		};
 *	   SpiChannelClockDivider spiChannelClockDivider = {div4, 2};
 *	   spiChannelConfig(Spi0, Ch7, Mode0, &spiChannelClockDivider, &spiPinMapping);
 * @endcode
 *
 */
extern void spiChannelConfig(SpiModule spiModule, SpiChannel spiChannel, SpiMode spiMode, SpiChannelClockDivider *spiChannelClockDivider, SpiPinsMapping *spiPinsMapping);

/**@brief Send and receive bytes in full-duplex mode.
 *
 * @param spiModule module
 * @param spiChannel channel
 * @param spiShiftMode data shift mode
 * @param wbuf pointer to array with data to be sent
 * @param rbuf pointer to array to write received data
 * @param len size of arrays
 * @return None
 *
 * Coding example:
 * @code
 *	uint8 wbuf[4] = {0x05, 0x00, 0xA0, 0xAA};
 *	uint8 rbuf[4];
 *	spiExchange(Spi0, Ch7, MsbFirst, wbuf, rbuf, 4);
 * @endcode
 *
 */
extern void spiExchange(SpiModule spiModule, SpiChannel spiChannel, SpiShiftMode spiShiftMode, void *wbuf, void *rbuf, uint16 len);

#endif /* SPI_H_ */
